import pandas as pd
import numpy as np
import math


class BusbarResistanceAnalysis(object):

    @staticmethod
    def fill_missing_rows(results_df, names, prefix, index):
        results_index_df = results_df[results_df[f'{prefix}_class'] == index]
        names_missing = set(names) - set(results_index_df[f'{prefix}_name'])
        if len(names_missing) > 0:
            df_missing = pd.DataFrame(list(names_missing), columns=[f'{prefix}_name'])
            df_missing[f'{prefix}_class'] = index
            return results_index_df.append(df_missing, ignore_index=True, sort=False)
        else:
            return results_index_df

    @staticmethod
    def merge_inj_and_sb(results_inj_df, results_sb_df, prefix):
        results_df = results_inj_df.merge(results_sb_df[[f'{prefix}_name', f'{prefix}_mean', f'{prefix}_std']],
                                          on=[f'{prefix}_name'])
        results_df = results_df.sort_values(by=[f'{prefix}_name'])
        # update column names
        old_cols = [f'{prefix}_mean_x', f'{prefix}_std_x', f'{prefix}_mean_y', f'{prefix}_std_y']
        new_cols = [f'{prefix}_mean_inj', f'{prefix}_std_inj', f'{prefix}_mean_sb', f'{prefix}_std_sb']
        results_df.rename(columns=dict(zip(old_cols, new_cols)), inplace=True)
        # drop class columns
        results_df.drop(columns=f'{prefix}_class', inplace=True)
        return results_df

    @staticmethod
    def divide(dividend, divisor):
        if divisor == 0:
            return math.nan
        else:
            return dividend / divisor

    @staticmethod
    def convert_into_row(u_res_i_meas_results_df, U_RES_names, i_meas_results_df, I_MEAS_devices, query_timing_values,
                         QUERY_TIMING_NAMES, index):
        values = np.concatenate([u_res_i_meas_results_df['Resistance'].values,
                                 u_res_i_meas_results_df['U_RES_mean_inj'].values,
                                 u_res_i_meas_results_df['U_RES_std_inj'].values,
                                 u_res_i_meas_results_df['U_RES_mean_sb'].values,
                                 u_res_i_meas_results_df['U_RES_std_sb'].values,
                                 i_meas_results_df['I_MEAS_mean_inj'].values,
                                 i_meas_results_df['I_MEAS_std_inj'].values,
                                 i_meas_results_df['I_MEAS_mean_sb'].values,
                                 i_meas_results_df['I_MEAS_std_sb'].values,
                                 query_timing_values])

        R_RES = [name.replace('U_RES', 'R_RES') for name in U_RES_names]
        U_RES_mean_inj = [f'{name}_mean_inj' for name in U_RES_names]
        U_RES_std_inj = [f'{name}_std_inj' for name in U_RES_names]
        U_RES_mean_sb = [f'{name}_mean_sb' for name in U_RES_names]
        U_RES_std_sb = [f'{name}_std_sb' for name in U_RES_names]

        I_MEAS_mean_inj = [f'{name}:I_MEAS_mean_inj' for name in I_MEAS_devices]
        I_MEAS_std_inj = [f'{name}:I_MEAS_std_inj' for name in I_MEAS_devices]
        I_MEAS_mean_sb = [f'{name}:I_MEAS_mean_sb' for name in I_MEAS_devices]
        I_MEAS_std_sb = [f'{name}:I_MEAS_std_sb' for name in I_MEAS_devices]

        COLUMN_NAMES = R_RES + U_RES_mean_inj + U_RES_std_inj + U_RES_mean_sb + U_RES_std_sb + \
                       I_MEAS_mean_inj + I_MEAS_std_inj + I_MEAS_mean_sb + I_MEAS_std_sb + QUERY_TIMING_NAMES

        return pd.DataFrame(data=[values], columns=COLUMN_NAMES, index=[index])