import pyspark.sql.functions as func
from pyspark.sql.functions import col
from cern.nxcals.pyquery.builders import *
import pandas as pd


class BusbarResistanceQuery(object):

    @staticmethod
    def query_current_mean_avg(system, devices, property_name, signal, t_start_nxcals, t_end_nxcals, translate_udf,
                               spark):
        # prepare NXCALS query
        i_meas = DevicePropertyQuery.builder(spark).system(system) \
            .startTime(t_start_nxcals).endTime(t_end_nxcals)

        # append device, property pairs
        for device in devices:
            i_meas = i_meas.entity().device(device).property(property_name)

        # build dataset
        i_meas_results = i_meas.buildDataset().select("acqStamp", signal, "device")

        # execute a query
        i_meas_results_ds = i_meas_results.withColumn("class", translate_udf(col("acqStamp"))).filter(
            col("class") != -1). \
            groupBy("device", "class").agg(func.mean(signal), func.stddev(signal)).collect()

        return pd.DataFrame(i_meas_results_ds,
                            columns=['I_MEAS_name', 'I_MEAS_class', 'I_MEAS_mean', 'I_MEAS_std']).sort_values(
            by=['I_MEAS_class'])

    @staticmethod
    def query_voltage_mean_avg(system, U_RES_names, t_start_nxcals, t_end_nxcals, translate_udf, spark):
        index = 0
        MAX_LIST_SIZE = 1000
        u_res_results_df_all = pd.DataFrame(columns=['U_RES_name', 'U_RES_class', 'U_RES_mean', 'U_RES_std'])
        while index < len(U_RES_names):
            u_res = VariableQuery.builder(spark).system(system) \
                .startTime(t_start_nxcals).endTime(t_end_nxcals)

            for U_RES_name in U_RES_names[index:index + MAX_LIST_SIZE]:
                u_res = u_res.variable(U_RES_name)

            u_res_results = u_res.buildDataset().select("nxcals_timestamp", 'nxcals_value', 'nxcals_variable_name')

            u_res_results_ds = u_res_results.withColumn("class", translate_udf(col("nxcals_timestamp"))).filter(
                col("class") != -1). \
                groupBy("nxcals_variable_name", "class").agg(func.mean('nxcals_value'),
                                                             func.stddev('nxcals_value')).collect()

            u_res_results_df_temp = pd.DataFrame(u_res_results_ds, columns=u_res_results_df_all.columns)
            u_res_results_df_all = pd.concat([u_res_results_df_all, u_res_results_df_temp], axis=0)

            index += MAX_LIST_SIZE

        return u_res_results_df_all
