import matplotlib.pyplot as plt

from .DiodeLeadResistanceAnalysis import DiodeLeadResistanceAnalysis
from .DiodeLeadResistanceQuery import DiodeLeadResistanceQuery
from ...Time import Time


class DiodeLeadResistancePlot(object):

    @staticmethod
    def plot_current_voltage_cals(magnet, circuit_name, u_diode, timestamp):
        magnet_type, I_MEAS, U_DIODE = DiodeLeadResistanceQuery.get_magnet_type_i_meas_u_diode(magnet, circuit_name)

        title = "Magnet: {}, Time Stamp: {}, I_MEAS(t) and U_DIODE(t)".format(magnet, Time.convertToString(timestamp)[:23])
        ax1 = u_diode[I_MEAS].plot(title=title,figsize=(15, 7.5), legend=True, lw=2, color='C0')
        ax1.title.set_size(20)
        ax1.set_xlabel("time, [s]", fontsize=20)
        ax1.tick_params(axis='both', which='major', labelsize=15)
        ax1.legend([I_MEAS], loc='lower left')
        ax1.set_ylabel("I_MEAS [A]", color='C0', fontsize=20)
        ax1.grid()

        ax2 = ax1.twinx()
        u_diode[U_DIODE].plot(ax=ax2, legend=True, lw=2, color='C2')
        ax2.set_ylabel("U_DIODE [V]", color='C2', fontsize=20)
        ax2.tick_params(axis='both', which='major', labelsize=15)
        plt.show()

    @staticmethod
    def plot_current_resistance_cals(magnet, circuit_name, u_diode, timestamp):
        magnet_type, I_MEAS, U_DIODE = DiodeLeadResistanceQuery.get_magnet_type_i_meas_u_diode(magnet, circuit_name)

        title = "Magnet: {}, Time Stamp: {}, I_MEAS(t) and R_DIODE_LEADS(t)".format(magnet,
                                                                              Time.convertToString(timestamp)[:23])
        ax1 = u_diode[I_MEAS].plot(title=title, figsize=(15, 7.5), legend=None, lw=2, color='C0')
        ax1.title.set_size(20)
        ax1.set_xlabel("time, [s]", fontsize=20)
        ax1.tick_params(axis='both', which='major', labelsize=15)
        ax1.legend([I_MEAS], loc='lower left')
        ax1.set_ylabel("I_MEAS [A]", color='C0', fontsize=20)
        ax1.grid()

        ax2 = ax1.twinx()
        u_diode['RES_filtered'].plot(ax=ax2, legend=True, lw=2, color='C2')
        ax2.set_ylabel("R_DIODE_LEADS(t) (Calculated) [uOhm]", color='C2', fontsize=20)
        ax2.tick_params(axis='both', which='major', labelsize=15)
        ax2.legend(['R_DIODE_LEADS(t)'], loc='upper right')

        x_text = 200 if magnet_type == "MB" else 100
        res_max_cals, t_res_max_cals, i_res_max_cals = DiodeLeadResistanceAnalysis.extract_res_max_time_current_cals(
            u_diode, I_MEAS)
        plot_text = ('r_max = {:.1f} uOhm\ni_max = {} A\nt_max = {} s'.
                    format(res_max_cals, int(i_res_max_cals), int(t_res_max_cals)))
        ax2.annotate(plot_text, xytext=(x_text, res_max_cals / 2), xy=(t_res_max_cals, res_max_cals),
                     arrowprops=dict(facecolor='white', shrink=0.05), fontsize=20)
        plt.show()

    @staticmethod
    def plot_current_voltage_pm(magnet_name, res_df, timestamp_nqps):
        title = "Magnet: {}, Time Stamp: {}, IAB.I_A(t) and U_DIODE(t)".format(magnet_name,
                                                                               Time.convertToString(timestamp_nqps)[
                                                                               :23])

        ax1 = res_df['IAB.I_A'].plot(figsize=(15, 10), title=title, legend=True, lw=2, xlim=(-0.2, 1.0), color='C0')
        ax1.title.set_size(20)
        ax1.set_xlabel("time, [s]", fontsize=20)
        ax1.tick_params(axis='both', which='major', labelsize=15)
        ax1.legend(['IAB.I_A'], loc='lower left')
        ax1.set_ylabel("IAB.I_A [A]", color='C0', fontsize=20)
        ax1.grid()

        ax2 = ax1.twinx()
        res_df['U_DIODE_A'].plot(ax=ax2, legend=None, lw=2, xlim=(-0.2, 1.0), color='C2')
        res_df['U_DIODE_B'].plot(ax=ax2, legend=None, lw=2, xlim=(-0.2, 1.0), color='C3')
        res_df['U_REF'].plot(ax=ax2, legend=None, lw=2, xlim=(-0.2, 1.0), color='C4')
        ax2.set_ylabel("U_DIODE [V]", color='C2', fontsize=20)
        ax2.legend(['U_DIODE_A', 'U_DIODE_B', 'U_REF'], loc='upper right')
        ax2.tick_params(axis='both', which='major', labelsize=15)
        plt.show()

    @staticmethod
    def plot_current_resistance_pm(circuit_name, magnet_name, res_df, timestamp_nqps):
        title = "Magnet: {}, Time Stamp: {}, IAB.I_A(t) and R_DIODE_LEADS(t)".format(magnet_name, Time.convertToString(
            timestamp_nqps)[:23])
        ax1 = res_df['IAB.I_A'].plot(title=title, legend='True', grid='True', color='C0')
        ax1.title.set_size(20)
        ax1.set_xlabel("time, [s]", fontsize=20)
        ax1.tick_params(axis='both', which='major', labelsize=15)
        ax1.legend(['IAB.I_A'], loc='lower left')
        ax1.set_ylabel("IAB.I_A [A]", color='C0', fontsize=20)

        ax2 = ax1.twinx()
        res_df['RES'].plot(ax=ax2, figsize=(15, 10), legend='True', color='C2')
        ax2.legend(['R_DIODE_LEADS'], loc='upper right')
        ax2.tick_params(axis='both', which='major', labelsize=15)
        ax2.set_ylabel('R_DIODE_LEADS(t) (Calculated) [uOhm]', color='C2', fontsize=20)

        x_text = 0.5 if "RB" in circuit_name else 1
        #
        res_max_pm, t_res_max_pm, i_res_max_pm = DiodeLeadResistanceAnalysis.extract_res_max_time_current_pm(res_df)

        plot_text = ('res_max = {:.1f} uOhm\ni_res_max = {} A\nt_res_max = {:.3f} s'.
                     format(res_max_pm, int(i_res_max_pm), t_res_max_pm))
        ax2.annotate(
            plot_text, xytext=(x_text, res_max_pm * 1e-6 / 2), xy=(t_res_max_pm, res_max_pm),
            arrowprops=dict(facecolor='white', shrink=0.05), fontsize=15)
        ax2.tick_params(axis='both', which='major', labelsize=15)
        plt.show()