from ...post_mortem.PmDbRequest import PmDbRequest
from ...dbsignal.Signal import Signal


class EnergyExtractionQuery(object):

    @staticmethod
    def query_u_dump_res(metadata_ee, timestamp_fgc):
        # find pm events
        event_source_timestamp_ee = PmDbRequest.findEvents('*', metadata_ee['system'], metadata_ee['className'],
                                                           startTime=timestamp_fgc, duration=[(1, 's'), (2, 's')])
        if len(event_source_timestamp_ee) != 2:
            raise ValueError("EE PMs were not found!")

        # metadata
        U_DUMP_RES = "U_DUMP_RES"
        # EE1
        source_ee_1 = event_source_timestamp_ee[0][0]
        timestamp_ee_1 = event_source_timestamp_ee[0][1]
        u_dump_res_1_df = Signal().read('pm', system=metadata_ee['system'], className=metadata_ee['className'],
                                        source=source_ee_1, signalName=U_DUMP_RES, eventTime=timestamp_ee_1)
        u_dump_res_1_sync_df = u_dump_res_1_df.set_index(((u_dump_res_1_df.index.values) - timestamp_fgc) / 1e+9)
        # EE2
        source_ee_2 = event_source_timestamp_ee[1][0]
        timestamp_ee_2 = event_source_timestamp_ee[1][1]
        u_dump_res_2_df = Signal().read('pm', system=metadata_ee['system'], className=metadata_ee['className'],
                                        source=source_ee_2, signalName=U_DUMP_RES, eventTime=timestamp_ee_2)
        u_dump_res_2_sync_df = u_dump_res_2_df.set_index(((u_dump_res_2_df.index.values) - timestamp_fgc) / 1e+9)

        return u_dump_res_1_sync_df, u_dump_res_2_sync_df, [timestamp_ee_1, timestamp_ee_2]

    @staticmethod
    def query_t_res_body(metadata_ee, T_RES_BODY, timestamp_fgc):
        event_source_timestamp_ee = PmDbRequest.findEvents('*', metadata_ee['system'], metadata_ee['className'],
                                                           startTime=timestamp_fgc, duration=[(10, 's'), (500, 's')])
        t_res_body_dfs = []
        for source_ee, timestamp_ee in event_source_timestamp_ee:
            for T_RES_BODY_el in T_RES_BODY:
                t_res_body_df = Signal().read('pm', append=True, system=metadata_ee['system'],
                                              className=metadata_ee['className'], source=source_ee,
                                              signalName=T_RES_BODY_el.replace('%EE%', source_ee),
                                              eventTime=timestamp_ee)

                t_res_body_sync_df = t_res_body_df.set_index(((t_res_body_df.index.values) - timestamp_fgc) / 1e+9)
                t_res_body_sync_df = t_res_body_sync_df.drop(t_res_body_sync_df.index[0:200])
                t_res_body_dfs.append(t_res_body_sync_df)
        return t_res_body_dfs
