from ...dbsignal.Signal import Signal
from ...dbsignal.SignalUtilities import SignalUtilities
from ...metadata.SignalMetadata import SignalMetadata
from ...post_mortem.PmDbRequest import PmDbRequest


class CurrentLeadsQuery(object):
    @staticmethod
    def query_u_leads(circuit_type, circuit_name, timestamp_fgc, *, leads_type):


        # check if leads type is either LEADS_EVEN or LEADS_ODD
        if not any([leads_type == lt for lt in ['LEADS_EVEN', 'LEADS_ODD']]):
            raise KeyError('Leads type {} not in {}!'.format(leads_type, ['LEADS_EVEN', 'LEADS_ODD']))

        # Find PM events
        metadata_leads = SignalMetadata.getCircuitVariableDatabaseMetadata(circuit_type, circuit_name, leads_type, 'PM')
        source_timestamp_leads = PmDbRequest.findEvents(source=metadata_leads['source'], system=metadata_leads['system'],
                                                        className=metadata_leads['className'],
                                                        startTime=timestamp_fgc, duration=[(10, 's'), (400, 's')])

        if len(source_timestamp_leads) != 1:
            raise Exception('There should be only 1 PM entry, found: {}'.format(len(source_timestamp_leads)))

        # Query PM signal
        U_LEADS = SignalMetadata.getVariableName(circuit_type, circuit_name, leads_type, 'PM', ['U_RES', 'U_HTS'])
        source_leads = source_timestamp_leads[0][0]
        timestamp_leads = source_timestamp_leads[0][1]

        u_leads_signal = Signal()
        for U_LEADS_el in U_LEADS:
            u_leads_signal.read('pm', append=True, source=source_leads, system=metadata_leads['system'],
                                className=metadata_leads['className'],
                                signalName=U_LEADS_el, eventTime=timestamp_leads)

        u_leads_df = u_leads_signal.getSynchronizedTimeDataframe(timestamp_leads)
        return SignalUtilities.convertIndicesToSeconds(u_leads_df)