from tqdm import tqdm

from ...metadata.SignalMetadata import SignalMetadata
from ...dbsignal.Signal import Signal


class NqpsQuery(object):

    @staticmethod
    def query_nqps_voltage(circuit_type, circuit_name, timestamp_fgc, *, system, signal, prefix='', ldb):
        NQPS_SIGNAL_template = SignalMetadata.getVariableName(circuit_type, circuit_name, system, 'CALS', signal)
        name, wildcards = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit_type, circuit_name, system, 'CALS',
                                                                             signal)
        NQPS_SIGNAL = [NQPS_SIGNAL_template.replace('%MAGNET%', prefix + wildcard) for wildcard in wildcards]

        nqps_signal_dfs = []
        for i in tqdm(range(len(NQPS_SIGNAL)), desc='Querying {}'.format(system)):
            nqps_signal_df = Signal().read('cals', name=NQPS_SIGNAL[i], startTime=(timestamp_fgc, 'ns'),
                                           duration=[(10, 's'), (500, 's')], ldb=ldb)
            nqps_signal_sync_df = nqps_signal_df.set_index((nqps_signal_df.index.values - timestamp_fgc * 1e-9))
            nqps_signal_dfs.append(nqps_signal_sync_df)

        return nqps_signal_dfs