import math
import pandas as pd
import requests

from ....metadata.MappingMetadata import MappingMetadata
from ....Time import Time

class IqdsAnalysis(object):

    @staticmethod
    def analyze_qds_trigger(u_qds_df):

        st_qds_sub_df = u_qds_df[(u_qds_df.index > u_qds_df.index[10]) &
                                 (u_qds_df['ST_MAGNET_OK'] == False)]

        if len(st_qds_sub_df) > 0:
            t_ST_MAGNET_OK = st_qds_sub_df.index[0]
            t_ST_NQD0 = u_qds_df[(u_qds_df.index > u_qds_df.index[10]) &
                                 (u_qds_df.index < t_ST_MAGNET_OK) &
                                 (u_qds_df['ST_NQD0'] == True)].index[-1]

            t_ST_NQD0 = u_qds_df.index[u_qds_df.index.get_loc(t_ST_NQD0) + 1]
            u_ST_NQD0 = u_qds_df[u_qds_df.index == t_ST_NQD0]['U_QS0'].values[0]

            if min(abs(u_qds_df['U_QS0'])) <= 0.01:
                u_qds_before_quench_df = u_qds_df[u_qds_df.index < t_ST_NQD0]
                t_start_quench = u_qds_before_quench_df[
                    abs(u_qds_before_quench_df['U_QS0'] - u_qds_before_quench_df['U_QS0'].values[0]) <= 0.01].index[-1]
                t_start_quench = u_qds_df.index[u_qds_df.index.get_loc(t_start_quench) - 1]
            else:
                t_start_quench = 0

            u_start_quench = u_qds_df.loc[
                u_qds_df.index[u_qds_df.index.get_loc(t_start_quench, method='nearest')], 'U_QS0']

            du_dt = (u_ST_NQD0 - u_start_quench) / (t_ST_NQD0 - t_start_quench)

            t_delay_qh_trigger = -t_ST_MAGNET_OK
            analysis_comment = ''
        else:
            t_ST_MAGNET_OK = u_qds_df.index[-1] - 0.075
            t_ST_NQD0 = u_qds_df.index[-1] - 0.075
            t_start_quench = u_qds_df.index[0] + 0.025
            u_start_quench = math.nan
            du_dt = 0
            t_delay_qh_trigger = math.nan
            analysis_comment = 'Seems, the heaters were triggered by nQPS'

        COLUMN_NAMES = ["t_ST_MAGNET_OK [s]", "t_ST_NQD0 [s]", "u_ST_NQD0 [V]", "t_start_quench [s]", "du_dt [V/s]",
                        "t_delay_qh_trigger [s]", "analysis_comment"]
        value = [t_ST_MAGNET_OK, t_ST_NQD0, u_start_quench, t_start_quench, du_dt, t_delay_qh_trigger, analysis_comment]
        return pd.DataFrame(columns=COLUMN_NAMES, data=[value])


    @staticmethod
    def create_analysis_report(circuit_type, circuit_name, metadata_qds, event_source_timestamp_qds, i_meas_df, timestamp_fgc):
        COLUMN_NAMES = ["Circuit", "magnet", "timeStampPM", "dateTimePM", "qCurrent", "A/B", "nQPS", "iQPS/nQPS",
                        "quenchType", "qPolarity", "dU_QS0/dt", "Rmax [uOhm]", "I_MEAS", "IEARTH", "EE", "EE_TEMP",
                        "qHeater", "vFeelers", "comments", "doneBy"]
        report_qds = pd.DataFrame(columns=COLUMN_NAMES)
        index = 1
        for source_qds, timestamp_qds in event_source_timestamp_qds:
            t_quench = (timestamp_qds - timestamp_fgc) * 1e-9
            i_quench = int(i_meas_df.reindex([t_quench], method='nearest', tolerance=.1)['I_MEAS'])

            print('{}/{}: Querying and analysing quench detection signals of {} on {}, {}'.format(index, len(
                event_source_timestamp_qds), source_qds, Time.convertToString(timestamp_qds), timestamp_qds))

            #ToDo It would be nice to add this query to the library
            pm_request = "http://pm-api-pro/v2/pmdata?system=QPS&className=" + metadata_qds[
                'className'] + "&source=" + source_qds + "&TimestampInNanos=" + str(
                timestamp_qds) + "&pageSize=1&pageIndex=0"
            resp = requests.get(pm_request)
            board_type = resp.json()["content"][0]["namesAndValues"][1]["value"]
            n_qps = MappingMetadata.getCrateNameFromMagnetName(circuit_type, source_qds)

            values = [circuit_name, source_qds, timestamp_qds, Time.convertToString(timestamp_qds)[:23], i_quench,
                      board_type, n_qps, '?', '?', '?', 0, 0, '', '', '', '', '', '', '', '']
            report_qds.loc[len(report_qds.index) + 1] = values
            index += 1

        return report_qds.drop_duplicates('magnet')