from ....metadata.MappingMetadata import MappingMetadata
from ....metadata.SignalMetadata import SignalMetadata
from ....dbsignal.SignalUtilities import SignalUtilities
from ....dbsignal.Signal import Signal


class NqdsQuery(object):

    @staticmethod
    def query_u_diode_st_magnet_ok_pm_nqds(circuit_type, circuit_name, source_nqps, timestamp_nqps):
        U_DIODE_RB = SignalMetadata.getVariableName(circuit_type, circuit_name, 'DIODE_{}'.format(circuit_type), 'PM',
                                                    'U_DIODE_{}'.format(circuit_type))
        U_REF_N1 = SignalMetadata.getVariableName(circuit_type, circuit_name, 'DIODE_{}'.format(circuit_type), 'PM',
                                                  'U_REF_N1')
        metadata_diode = SignalMetadata.getCircuitVariableDatabaseMetadata(circuit_type, circuit_name,
                                                                           'DIODE_{}'.format(circuit_type), 'PM')

        magnet_names = MappingMetadata.getMagnetNamesFromCrateName(circuit_type, source_nqps)

        # U_DIODE
        u_nqps_signal = Signal()
        for magnet_name in magnet_names:
            u_nqps_signal.read('pm', append=True, system=metadata_diode['system'], className=metadata_diode['className'],
                               source=source_nqps,
                               signalName=U_DIODE_RB.replace('%MAGNET%', magnet_name), eventTime=timestamp_nqps)
        try:
            u_nqps_signal.read('pm', append=True, system=metadata_diode['system'], className=metadata_diode['className'],
                               source=source_nqps,
                               signalName=U_REF_N1.replace('%QPS_CRATE%', source_nqps).replace('%CIRCUIT%', circuit_name),
                               eventTime=timestamp_nqps)
        except:
            print('U_REF_N1 signal not present for crate {}'.format(source_nqps))

        u_nqps_df = u_nqps_signal.getSynchronizedTimeDataframe(timestamp_nqps)
        u_nqps_df = SignalUtilities.convertIndicesToSeconds(u_nqps_df)

        # ST_MAGNET_OK_SQRB
        ST_MAGNET_OK_SQRB = "%MAGNET%:ST_MAGNET_OK_SQRB"
        st_nqps_signal = Signal()
        for magnet_name in magnet_names:
            st_nqps_signal.read('pm', append=True, system=metadata_diode['system'], className=metadata_diode['className'],
                                source=source_nqps,
                                signalName=ST_MAGNET_OK_SQRB.replace('%MAGNET%', magnet_name), eventTime=timestamp_nqps)

        st_nqps_df = st_nqps_signal.getSynchronizedTimeDataframe(timestamp_nqps)
        st_nqps_df = SignalUtilities.convertIndicesToSeconds(st_nqps_df)

        return u_nqps_df, st_nqps_df

    @staticmethod
    def query_u_diode_cals_nqds(circuit_type, circuit_name, source_nqps, timestamp_nqps, ldb):
        U_DIODE_RB = SignalMetadata.getVariableName(circuit_type, circuit_name, 'DIODE_{}'.format(circuit_type), 'CALS',
                                                    'U_DIODE_{}'.format(circuit_type))
        U_REF_N1 = SignalMetadata.getVariableName(circuit_type, circuit_name, 'DIODE_{}'.format(circuit_type), 'CALS',
                                                  'U_REF_N1')

        magnet_names = MappingMetadata.getMagnetNamesFromCrateName(circuit_type, source_nqps)

        # U_DIODE
        u_nqps_signal = Signal()
        for magnet_name in magnet_names:
            u_nqps_signal.read('cals', append=True, name=U_DIODE_RB.replace('%MAGNET%', magnet_name),
                               startTime=(timestamp_nqps, 'ns'), duration=[(2, 's'), (2, 's')], ldb=ldb)
        try:
            u_nqps_signal.read('cals', append=True,
                               name=U_REF_N1.replace('%QPS_CRATE%', source_nqps).replace('%CIRCUIT%', circuit_name),
                               startTime=(timestamp_nqps, 'ns'), duration=[(2, 's'), (2, 's')], ldb=ldb)
        except:
            print('U_REF_N1 signal not present for crate {}'.format(source_nqps))

        return u_nqps_signal.getSynchronizedTimeDataframe(timestamp_nqps * 1e-9)

    @staticmethod
    def query_nqps_pm_cals(circuit_type, circuit_name, source_nqps, timestamp_nqps, timestamp_qds, ldb):
        if abs(timestamp_nqps - timestamp_qds) < 2e9:
            try:
                u_nqps_df, st_nqps_df = NqdsQuery.query_u_diode_st_magnet_ok_pm_nqds(circuit_type, circuit_name,
                                                                                     source_nqps, timestamp_nqps)
            except:
                print(f'Could not query PM for {source_nqps} at {timestamp_nqps}, querying CALS instead')
                u_nqps_df = NqdsQuery.query_u_diode_cals_nqds(circuit_type, circuit_name, source_nqps, timestamp_qds,
                                                              ldb=ldb)
                st_nqps_df = None
        else:
            u_nqps_df = NqdsQuery.query_u_diode_cals_nqds(circuit_type, circuit_name, source_nqps, timestamp_qds,
                                                          ldb=ldb)
            st_nqps_df = None
        return u_nqps_df, st_nqps_df