from ipywidgets import HBox, Button, Label, IntProgress, Layout
import functools

from ...post_mortem.PmDbRequest import PmDbRequest
from ...Time import Time
from ...gui.pc.FgcBrowserAction import FgcBrowserAction
from ...metadata.SignalMetadata import SignalMetadata
from ...gui.BaseModule import BaseModule


class FgcPmSearchBaseModule(BaseModule):

    def __init__(self):
        self.find_btn = Button(
            description='Find FGC PM entries',
            tooltip='Click to find FGC PM entries between indicated dates',
            button_style='',  # 'success', 'info', 'warning', 'danger' or ''
        )
        self.find_btn.on_click(functools.partial(FgcPmSearchBaseModule.handle_find_button_event, self=self))

        # # Text field
        self.lbl_search = Label(value="Click button to search for FGC PM entries within the specified period of time")

        # # Progress bar
        self.progress_search = IntProgress(value=0, min=0, step=1, bar_style='success',
                                           orientation='horizontal', layout=Layout(width='20%', visibility='hidden'))

        self.widget = HBox(children=[self.find_btn, self.lbl_search, self.progress_search])

    def handle_find_button_event(b, self):
        self.mediator.notify(self, FgcBrowserAction.FIND_BUTTON_CLICKED)

    def find_fgc_pm_entries(self, start_time, end_time, circuit_type, circuit_name):

        metadata_fgc = SignalMetadata.getCircuitVariableDatabaseMetadata(circuit_type, circuit_name, 'PC', 'PM')

        quotient, remainder = Time.moduloDayInUnixTimestamp(start_time, end_time)
        eventDurationNanoSeconds = int(24 * 60 * 60 * 1e9)
        event_source_timestamp_fgc = []

        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = quotient
        self.progress_search.description = f'0/{quotient}'

        for day in range(quotient):
            startTimeTemp = Time.convertToUnixTimestamp(start_time) + int(day * eventDurationNanoSeconds)

            # Update GUI
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.convertToString(startTimeTemp))

            event_source_timestamp_fgc += PmDbRequest.findEvents(metadata_fgc['source'], metadata_fgc['system'],
                                                                 metadata_fgc['className'], startTime=startTimeTemp,
                                                                 duration=[(eventDurationNanoSeconds, 'ns')])
            self.progress_search.description = f'{day + 1}/{quotient}'
            self.progress_search.value = day + 1

        if remainder > 0:
            startTimeTemp = Time.convertToUnixTimestamp(end_time) - remainder
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.convertToString(startTimeTemp))
            event_source_timestamp_fgc += PmDbRequest.findEvents(metadata_fgc['source'], metadata_fgc['system'],
                                                                 metadata_fgc['className'], startTime=startTimeTemp,
                                                                 duration=[(remainder, 'ns')])

        self.lbl_search.value = 'Query completed.'

        return event_source_timestamp_fgc

    def get_circuit_name(self):
        return self.select_circuit_name_dropdown.value