from ipywidgets import Dropdown, Text, HBox, VBox, Select, Label

from ...Time import Time
from ...gui.pc.FgcBrowserAction import FgcBrowserAction
from ...gui.pc.FgcPmSearchBaseModule import FgcPmSearchBaseModule
from ...metadata.SignalMetadata import SignalMetadata
from ...gui.ModuleMediator import ModuleMediator
from ...gui.qh.DateTimeBaseModule import DateTimeBaseModule
from ...gui.qh.QhBrowserAction import QhBrowserAction


class FgcPmSearchModuleMediator(ModuleMediator):

    def __init__(self, date_time_module: DateTimeBaseModule, fgc_pm_search_module: FgcPmSearchBaseModule,
                 circuit_type='RB'):
        self._circuit_type = circuit_type
        # First row
        self.select_circuit_name_dropdown = Dropdown(
            options=SignalMetadata.getCircuitNames(circuit_type),
            description='Circuit name:',
            disabled=False,
        )

        self._date_time_module = date_time_module
        self._date_time_module.mediator = self

        self.done_by_txt = Text(placeholder='Your name', description='Done by:')
        first_row = HBox(children=[self.select_circuit_name_dropdown, self._date_time_module.widget, self.done_by_txt])

        # Second row
        self._fgc_pm_search_module = fgc_pm_search_module
        self._fgc_pm_search_module.mediator = self
        second_row = fgc_pm_search_module.widget

        # Third row
        self.fgc_pm_sel = Select(
            options=[],
            rows=10,
            disabled=False
        )
        third_row = HBox(children=[Label(value='FGC PM Entries:'), self.fgc_pm_sel])

        # Final GUI
        self.widget = VBox(children=[first_row, second_row, third_row])

    def notify(self, sender: object, event: str) -> None:
        start_date_time = self._date_time_module.get_start_date_time()
        end_date_time = self._date_time_module.get_end_date_time()
        circuit_type = self.get_circuit_type()
        circuit_name = self.get_circuit_name()

        if event == QhBrowserAction.DATE_CHANGED:
            pass

        elif event == FgcBrowserAction.FIND_BUTTON_CLICKED:
            self.fgc_pm_entries = self._fgc_pm_search_module.find_fgc_pm_entries(start_date_time, end_date_time,
                                                                                 circuit_type, circuit_name)
            self.fgc_pm_sel.options = [Time.convertToString(el[1]) for el in self.fgc_pm_entries]

        else:
            raise KeyError(f'Action {event} not supported!')

    def get_author(self):
        return self.done_by_txt.value

    def get_fgc_pm_entry(self):
        if len(self.fgc_pm_sel.options) == 0:
            raise ValueError(
                'The list of FGC PM Entries should contain at least one element. Please adjust circuit name as well as start and end date.')
        else:
            return Time.convertToUnixTimestamp(self.fgc_pm_sel.value)

    def get_circuit_type(self):
        return self._circuit_type

    def get_circuit_name(self):
        return self.select_circuit_name_dropdown.value